/*****************************************************************************/
/* File name: ActionCreate.java                                              */
/* Purpose: to keep an option and a corresponding flag for Jtar              */
/* Last modified: 12.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* This software is distributed under the terms                              */
/* of the GNU General Public License as published                            */
/* by the Free Software Foundation; either version 2, or (at your option)    */
/* any later version.                                                        */
/* See file COPYING for your rights (GNU GPL)                                */
/*****************************************************************************/

package jtar;

import java.io.*;
import java.util.*;
import general.*;

class

ActionCreate

                   extends Action

     implements jtar.Constants, general.Constants

{

  private final static String RCSID = "$Header: /d/JTAR03/jtar/RCS/ActionCreate.java,v 1.2 1998/12/13 17:19:16 vklimov Exp $";


  String  dirNameToPrepend = null;
  boolean currentlyPrepending = false;
  int nameIndex = 0;
  int nNames = 0;

  ActionCreate( char    option,
                boolean flag   )
/*****************************************************************************/
/* Function name: ActionCreate                                               */
/* File name: ActionCreate.java                                              */
/* Purpose: constructor for the ActionCreate object                          */
/* Parameters: option - 't', 'x', 'c' etc                                    */
/*             flag - corresponding flag                                     */
/* Returns:                                                                  */
/* Last modified: 11.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                    /* ActionCreate */
/*---------------------------------------------------------------------------*/

      super( option, flag );

    }
/*end of ActionCreate*/


  void
  collectAndSortNames()
/*****************************************************************************/
/* Function name: collectAndSortNames                                        */
/* File name: ActionCreate.java                                              */
/* Purpose: to collect and sort file names                                   */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 12.12.98                                                   */
/* Author of the Java translation: Victor Klimov, 1998                       */
/*****************************************************************************/
    {
                                              /* collectAndSortNames */
/*---------------------------------------------------------------------------*/

    }
/*end of collectAndSortNames*/


  void
  blankNameList()
/*****************************************************************************/
/* Function name: blankNameList                                              */
/* File name: ActionCreate.java                                              */
/* Purpose: to init the file name list to its original form                  */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 12.12.98                                                   */
/* Author of the Java translation: Victor Klimov, 1998                       */
/*****************************************************************************/
    {
                                                    /* blankNameList */
/*---------------------------------------------------------------------------*/

    }
/*end of blankNameList*/


  String
  nameFromList()
/*****************************************************************************/
/* Function name: nameFromList                                               */
/* File name: ActionCreate.java                                              */
/* Purpose: to return a new file name to dump                                */
/* Parameters:                                                               */
/* Returns: the file name                                                    */
/* Last modified: 12.12.98                                                   */
/* Author of the Java translation: Victor Klimov, 1998                       */
/*****************************************************************************/
    {
      String fileName = null;
                                                     /* nameFromList */
/*---------------------------------------------------------------------------*/

      return fileName;

    }
/*end of nameFromList*/


  String
  nameNext( Vector  commandLineOptionsVector,
             boolean changeDirs                   )
/*****************************************************************************/
/* Function name: nameNext                                                   */
/* File name: ActionCreate.java                                              */
/* Purpose: to get next file name from the list of options                   */
/* Parameters: commandLineOptionsVector - vector, containig strings          */
/*                                 of command line arguments to Jtar         */
/*             changeDirs                                                    */
/* Returns: the file name or null if no more names left                      */
/* Last modified: 12.12.98                                                   */
/* Author of the java version: Victor Klimov, 1998                           */
/*****************************************************************************/
    {
      String source = null;
      StringBuffer fileNameWithPrefix;
      File   dirFile;
      boolean beginToPrependDirName = false;

                                                         /* nameNext */
/*---------------------------------------------------------------------------*/

      nNames = commandLineOptionsVector.size();

      while( nameIndex < nNames )
        {
          source = (String)
            commandLineOptionsVector.
              elementAt(nameIndex++);

          source =
            util.String.
              removeTrailingPathSeparator(
                source                     );

          if (beginToPrependDirName)
            {
              try
                {
                  dirFile = new File( source );
                  if ( dirFile.exists() )
                    {
                      dirNameToPrepend =
                        new String( source );
                      beginToPrependDirName = false;
                      currentlyPrepending = true;
                    }
                  else
                    Error.
                      reportAndExit(                            /* abort ! */
                        "directory \"" + source +
                        "\" does not exist, exiting" );
                /*end of if*/

                }
              catch( NullPointerException e)
                {
                  Error.
                    reportAndExit(                              /* abort ! */
                      e.getMessage() +
                      "\nnull directory name given as an option" );
                }
              catch( SecurityException e)
                {
                  Error.
                    reportAndExit(                              /* abort ! */
                      e.getMessage() );
                }
            /*end of try-catch*/
            }
          else /* previous vector element was not "-C" */
            {
              if ( changeDirs &&
                   source.equals(
                     COMMAND_LINE_OPTION_CHDIR_STRING )
                 )
                {
                  beginToPrependDirName = true;
                  currentlyPrepending = false;
                }
              else /* don't change dirs */
                {
                  source =
                    util.QuoteString.
                      unquoteString( source );
                  if (currentlyPrepending)
                    {
                      fileNameWithPrefix = new
                        StringBuffer( dirNameToPrepend );
                      fileNameWithPrefix.
                        append( File.separator );
                      fileNameWithPrefix.
                        append( source );
                      return fileNameWithPrefix.toString();     /* return ! */
                    }
                  else
                    return source;                              /* return ! */
                /*end of if*/
                }
            /*end of if on changeDirs*/
            }
        /*end of if on beginToPrependDirName*/
        }
    /*end of while-loop*/

      if ( (source != null) &&
           beginToPrependDirName)
        Error.
          reportAndExit(                              /* abort ! */
            "Missing file name after -C" );
      else
        ;
    /*end of if on null*/

      return source;                                 /* return ! */

    }
/*end of nameNext*/


  boolean
  action( Vector commandLineOptionsVector,
          Header dummyTarHeader,
          BufferRead dummyReadBuffer )
/*****************************************************************************/
/* Function name: action to create an archive                                */
/* File name: ActionCreate.java                                              */
/* Purpose: to create an archive                                             */
/* Parameters: commandLineOptionsVector - vector, containig strings          */
/*                                 of command line arguments to Jtar         */
/*             dummyTarHeader                                                */
/*             dummyReadBuffer                                               */
/* Returns: true - if everything is all right, false - otherwise             */
/* Last modified: 12.12.98                                                   */
/* Author of the Java translation: Victor Klimov, 1998                       */
/*****************************************************************************/
    {
      boolean readOK = true;
      String fileToDumpName;

      BufferWrite writeBuffer = new BufferWrite();
      Header tarHeader = new Header();
                                                     /* action */
/*---------------------------------------------------------------------------*/

      readOK =
        writeBuffer.open();

      if (!readOK)
        return false;                                  /* return ! */
      else
        ;
    /*end of if on readOK*/

      if (AllActions.incremental.flag)
        {
          /* to implement in the future */
        }
      else /* not incremental */
        {
          fileToDumpName =
            nameNext( commandLineOptionsVector,
                       true /* change dirs */        );

          while( fileToDumpName != null )
            {

              if (AllActions.verbose.flag)
                System.out.println(
                  "dumping file: " +
                    fileToDumpName    );
              else
                ;
            /*end of if on verbose*/

              writeBuffer.
                dumpFile( fileToDumpName,
                           true, /*top level call*/
                           writeBuffer,
                           tarHeader                );
              fileToDumpName =
                nameNext( commandLineOptionsVector,
                           true /* change dirs */        );
            }
        /*end of while-loop*/
        }
    /*end of if on incremental*/

      writeBuffer.writeEot();
      writeBuffer.close();

      return true;

    }
/*end of action*/


} /*end of class ActionCreate */

/*****************************************************************************/
/* End of file: ActionCreate.java                                            */
/*****************************************************************************/
