/*****************************************************************************/
/* File name: AllActions.java                                                */
/* Purpose: to keep flags and options for Jtar                               */
/* Last modified: 13.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* This software is distributed under the terms                              */
/* of the GNU General Public License as published                            */
/* by the Free Software Foundation; either version 2, or (at your option)    */
/* any later version.                                                        */
/* See file COPYING for your rights (GNU GPL)                                */
/*****************************************************************************/

package jtar;

import c.Stat;
import general.*;

final

class

AllActions

     implements jtar.Constants, general.Constants

{

  private final static String RCSID = "$Header: /d/JTAR10/jtar/RCS/AllActions.java,v 1.3 1998/12/20 00:11:17 vklimov Exp $";

  static ActionTable table  = new
                              ActionTable(
                                 COMMAND_LINE_OPTION_TABLE,false );
  static ActionCreate create  = new
                              ActionCreate(
                                 COMMAND_LINE_OPTION_CREATE,false );
  static ActionExtract extract = new
                              ActionExtract(
                                 COMMAND_LINE_OPTION_EXTRACT,false );
  static Action file = new
                              Action(
                                 COMMAND_LINE_OPTION_FILE,false );
  static Action gzip = new
                              Action(
                                 COMMAND_LINE_OPTION_GZIP,false );
  static Action verbose = new
                              Action(
                                 COMMAND_LINE_OPTION_VERBOSE,false );

  static Action incremental = new
                              Action(
                                 COMMAND_LINE_OPTION_INCREMENTAL,false );


                                            /* for the time being */

  static boolean useCompressProgram = false;  /* no compress yet    */
  static boolean numericOwner = false;
  static boolean blockNumberOption = false;
  static long newerMtime = Long.MIN_VALUE;
  static boolean exclude = false;
  static boolean multiVolume = false;
  static boolean showOmittedDirs = true;
  static boolean interactive = false;
  static boolean absoluteNames = false;
  static boolean backup = false;
  static boolean toStdout = false;
  static boolean keepOldFiles = true;


  static
  Action[]
    flagOptionsArray   = { (Action) table,
                           (Action) create,
                           (Action) extract
                         };

  static String archiveName = null;
  static String currentFileName = null;
  static String currentLinkName = null;

  static Action whatToActUpon = null;

  static int archiveFormat = DEFAULT_FORMAT;
  static int recordSize = DEFAULT_BLOCKING * BLOCKSIZE;

  static c.Stat currentStat = new c.Stat();

  /* never instantiate this class, all methods and variables are */
  /* class variables and and class methods                       */

 private
  AllActions()  /* no arguments */
   {                  /* no content   */
   }
/*end of AllActions*/


  static
  boolean

  setUp( String options )
/*****************************************************************************/
/* Function name: setUp                                                      */
/* File name: AllActions.java                                                */
/* Purpose: to scan the options string and to assign value to flags          */
/* Parameters: options - string of option letters specified on the command   */
/*                       line                                                */
/* Returns: true - when all the option letters are recognized,               */
/*          false - otherwise                                                */
/* Last modified: 12.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      int i, j;
      boolean optionFound = false;
      int optionsLength;
      int flagsLength;
      boolean allOptionLettersRecognized = true;
                                                          /* setUp */
/*---------------------------------------------------------------------------*/

      optionsLength = options.length();
      flagsLength = flagOptionsArray.length;

      for (i=0; i < optionsLength; i++)
        {

          optionFound = false;
          for (j=0; j < flagsLength; j++)
            {
              if (options.charAt(i) ==
                  flagOptionsArray[j].option)
                {
                  flagOptionsArray[j].flag = true;
                  optionFound = true;

                  if (AllActions.verbose.flag)
                    System.out.println( "option " + "\"" +
                                        options.charAt(i) +
                                        "\" recognized and flag assigned" );
                  else
                    ;
                /*end of if on AllActions.verbose.flag*/
                }
              else
                ;
            /*end of if*/
            }
        /*end of for-loop on j*/

          if (!optionFound &&
              (options.charAt(i) !=
               AllActions.verbose.option) &&
              (options.charAt(i) !=
               AllActions.file.option)
             )
            {

              allOptionLettersRecognized = false;
              if (AllActions.verbose.flag)
                System.out.println( "\noption " + "\"" +
                                    options.charAt(i) +
                                    "\" not recognized" );
              else
                ;
            /*end of if on AllActions.verbose.flag*/

            }
          else
            ;
        /*end of if on optionFound*/

        }
      /*end of for-loop on i*/

      return allOptionLettersRecognized;

    }
/*end of setUp*/


  static
  boolean

  optionsFitGoodTogetherSetMethods()
/*****************************************************************************/
/* Function name: optionsFitGoodTogetherSetMethods                           */
/* File name: AllActions.java                                                */
/* Purpose: to check whether there are any conflicts between options set     */
/* Parameters:                                                               */
/* Returns: true - when everything is OK                                     */
/*          false - if there are conflicting options                         */
/* Last modified: 12.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      int j;
      int nFlagsSet = 0;
      int flagsLength;
      boolean allOptionsAreJustFine = true;
                                   /* optionsFitGoodTogetherSetMethods */
/*---------------------------------------------------------------------------*/

      flagsLength = flagOptionsArray.length;

      for (j=0; j < flagsLength; j++)
        {
          if (flagOptionsArray[j].flag)
            {
              nFlagsSet++;
              whatToActUpon = flagOptionsArray[j];
              if (AllActions.verbose.flag)
                System.out.println( "flag set for option " +
                                    "\"" +
                                    flagOptionsArray[j].option +
                                    "\"" );
              else
                ;
            /*end of if on AllActions.verbose.flag*/
            }
          else
            ;
        /*end of if*/
        }
    /*end of for-loop on j*/

      if (nFlagsSet >
          N_MAX_MAIN_OPTIONS)
        {
          allOptionsAreJustFine = false;

          if (AllActions.verbose.flag)
            System.err.println( "\nconflicting options!" );
          else
            ;
        /*end of if on AllActions.verbose.flag*/
        }
      else
        ;
    /*end of if on optionFound*/

      return allOptionsAreJustFine;

    }
/*end of optionsFitGoodTogetherSetMethods*/


} /*end of AllActions */

/*****************************************************************************/
/* End of file: AllActions.java                                              */
/*****************************************************************************/

