/*****************************************************************************/
/* File name: Jtar.java                                                      */
/* Purpose: tar (GNU) translated into the Java language                      */
/*                                                                           */
/* Last modified: 20.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/* For the list of authors and contributors of the original C version        */
/* of the GNU tar see files AUTHORS and THANKS                               */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* This software is distributed under the terms                              */
/* of the GNU General Public License as published                            */
/* by the Free Software Foundation; either version 2, or (at your option)    */
/* any later version.                                                        */
/* See file COPYING for your rights (GNU GPL)                                */
/*****************************************************************************/

package jtar;


import java.util.*;
import general.*;


class

Jtar

     implements jtar.Constants, general.Constants

{

  private final static String RCSID = "$Header: /d/JTAR11/jtar/RCS/Jtar.java,v 1.5 1998/12/21 17:12:14 vklimov Exp $";


  final static String VERSION_NUMBER_STRING = "Version 1.1";

  final static String COMMAND_LINE_OPTION_LONG_PREFIX  = "--";
  final static String COMMAND_LINE_OPTION_LONG_HELP    = "help";
  final static String COMMAND_LINE_OPTION_LONG_VERSION = "version";
  final static int    COMMAND_LINE_ARCH_NAME_NUMBER    = 2;      /*from 1 on*/

  final static String HELP_EXAMPLE_ENVOCATION_PREFIX    = "java Jtar ";
  final static String HELP_ARCHIVE_NAME_SUFFIX          = " archive.tar";
  final static String HELP_EXAMPLE_DIR_NAME_SUFFIX      = " ./*";
  final static String HELP_EXAMPLE_EMPTY_PREFIX         =
                        "                             ";


  public
  static
  boolean

  scanForTheLongHelpOption( String[] args,
                            Vector commandLineOptionsVector  )
/*****************************************************************************/
/* Function name: scanForTheLongHelpOption                                   */
/* File name: Jtar.java                                                      */
/* Purpose: to scan for the help option                                      */
/* Parameters: args - arguments on the command line                          */
/*             commandLineOptionsVector - vector, containig strings          */
/*                               of command line arguments to Jtar           */
/* Returns: true - if the help option found, false - otherwise               */
/* Last modified: 12.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      int i;
      boolean continueToScanOptions = true;
      boolean helpOptionSpecified = false;

                                          /* scanForTheLongHelpOption */
/*---------------------------------------------------------------------------*/

      for ( i = 0;
            (continueToScanOptions &&
            (i < args.length));           i++)
        {
          helpOptionSpecified =
            args[i].equals( COMMAND_LINE_OPTION_LONG_PREFIX +
                            COMMAND_LINE_OPTION_LONG_HELP     );
          if (helpOptionSpecified)
            {
              continueToScanOptions = false;
              help();
            }
          else
            
            commandLineOptionsVector.addElement( args[i] );

        /*end of if on helpOptionSpecified*/
        }
    /*end of for-loop*/

      return helpOptionSpecified;

    }
/*end of scanForTheLongHelpOption*/



  public
  static
  boolean

  scanForTheLongVersionOption( Vector commandLineOptionsVector )
/*****************************************************************************/
/* Function name: scanForTheLongVersionOption                                */
/* File name: Jtar.java                                                      */
/* Purpose: to scan for the version option                                   */
/* Parameters: commandLineOptionsVector - vector, containig strings          */
/*                               of command line arguments to Jtar           */
/* Returns: true - if the version option is found, false - otherwise         */
/* Last modified: 20.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      int i;
      int nElementsInTheVector;
      boolean continueToScanOptions = true;
      boolean versionOptionSpecified = false;
      String option;

                                          /* scanForTheLongVersionOption */
/*---------------------------------------------------------------------------*/

      nElementsInTheVector =
        commandLineOptionsVector.size();

      for ( i = 0;
            (continueToScanOptions &&
            (i < nElementsInTheVector));  i++)
        {
          option = (String)
            commandLineOptionsVector.elementAt(i);
          versionOptionSpecified =
            option.equals( COMMAND_LINE_OPTION_LONG_PREFIX +
                           COMMAND_LINE_OPTION_LONG_VERSION  );
          if (versionOptionSpecified)
            {
              commandLineOptionsVector.removeElementAt(i); /* don't need it */
                                                           /* anymore       */
              continueToScanOptions = false;

              System.out.println( VERSION_NUMBER_STRING );
            }
          else
            ;
        /*end of if on versionOptionSpecified*/
        }
    /*end of for-loop*/

      return versionOptionSpecified;

    }
/*end of scanForTheLongVersionOption*/


  public
  static
  boolean

  checkForCommandLineOptionGzip( Vector commandLineOptionsVector )
/*****************************************************************************/
/* Function name: checkForCommandLineOptionGzip                              */
/* File name: Jtar.java                                                      */
/* Purpose: to analyze command line arguments                                */
/* Parameters: commandLineOptionsVector - vector, containig strings          */
/*                                 of command line arguments to Jtar         */
/* Returns: true - when the GZIP option is specified, false - otherwise      */
/* Last modified: 15.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      String options;
      int nChar;
                                  /* checkForCommandLineOptionGzip */
/*---------------------------------------------------------------------------*/

      options = (String)
        commandLineOptionsVector.firstElement();
      
      nChar = options.indexOf(
        COMMAND_LINE_OPTION_GZIP );

      if (nChar >= 0 )
        {
          options =
            util.String.
              removeAllSpecificChars(
                options,
                COMMAND_LINE_OPTION_GZIP );

          commandLineOptionsVector.
                    removeElementAt( 0 );
          commandLineOptionsVector.
            insertElementAt( (Object) options, 0 );

          AllActions.gzip.flag = true;
          return true;                                 /* return ! */
        }
      else
        return false;
    /*end of if*/

    }
/*end of checkForCommandLineOptionGzip*/


  public
  static
  boolean

  checkForCommandLineOptionFile( Vector commandLineOptionsVector )
/*****************************************************************************/
/* Function name: checkForCommandLineOptionFile                              */
/* File name: Jtar.java                                                      */
/* Purpose: to analyze command line arguments                                */
/* Parameters: commandLineOptionsVector - vector, containig strings          */
/*                                 of command line arguments to Jtar         */
/* Returns: true - when the file option is specified, false - otherwise      */
/* Last modified: 20.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      int nElementsInTheVector;
      String options;
      int length;
      char lastChar;
                                  /* checkForCommandLineOptionFile */
/*---------------------------------------------------------------------------*/

      nElementsInTheVector =
        commandLineOptionsVector.size();
      if (nElementsInTheVector <= 0)
        return false;                                  /* return ! */
      else
        ;
    /*end of if*/

      options = (String)
        commandLineOptionsVector.firstElement();
      length = options.length();
      lastChar = options.charAt( length-1 );

      if (lastChar == COMMAND_LINE_OPTION_FILE)
        {
          AllActions.file.flag = true;
          return true;                                 /* return ! */
        }
      else
        return false;
    /*end of if*/

    }
/*end of checkForCommandLineOptionFile*/


  public
  static
  void

  checkVerbose( String options,
                Vector commandLineOptionsVector )
/*****************************************************************************/
/* Function name: checkVerbose                                               */
/* File name: Jtar.java                                                      */
/* Purpose: to find out whether the verbose command line option              */
/*          is specified and if yes to set the corresponding internal flag   */
/* Parameters: options - the command line options string                     */
/*             commandLineOptionsVector - vector, containig strings          */
/*                                 of command line arguments to Jtar         */
/* Returns:                                                                  */
/* Last modified: 12.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      int i;
                                                    /* checkVerbose */
/*---------------------------------------------------------------------------*/

      if ( options.indexOf(
             AllActions.verbose.option) > 0 )    /* v found */
        {
          AllActions.verbose.flag = true;
          System.out.println( "verbose option is specified" );
          System.out.println( "following command line " +
                              "options are specified:"     );

          for (i=0;
               i < commandLineOptionsVector.size();
               i++)
            System.out.print( (String)
              commandLineOptionsVector.elementAt( i ) + " " );
        /*end of for-loop*/

          System.out.print( "\n" );

        }
      else
        AllActions.verbose.flag = false;
    /*end of if*/

    }
/*end of checkVerbose*/

  public
  static
  boolean

  checkAndGetArchiveName( Vector commandLineOptionsVector )
/*****************************************************************************/
/* Function name: checkAndGetArchiveName                                     */
/* File name: Jtar.java                                                      */
/* Purpose: to check whether a name of tar archive file is specified         */
/* Parameters: commandLineOptionsVector - vector, containig strings          */
/*                                 of command line arguments to Jtar         */
/* Returns: true - if the name is specified, false - otherwise               */
/* Last modified: 20.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      boolean nameSpecified = false;
                                            /* checkAndGetArchiveName */
/*---------------------------------------------------------------------------*/

      if (commandLineOptionsVector.size() <
          COMMAND_LINE_ARCH_NAME_NUMBER)             /* no archive name */
        {
          System.err.println( "\"" +
            java.lang.String.valueOf(
              COMMAND_LINE_OPTION_FILE ) +
            "\"" +
            " option: archive name is not specified\n" );
          help();
          nameSpecified = false;
        }
      else                         /* arch name found */
        {
          nameSpecified = true;
          AllActions.archiveName = (String)
            commandLineOptionsVector.elementAt(
                         COMMAND_LINE_ARCH_NAME_NUMBER-1 );

          try
            {                                  /* remove archive name */
              commandLineOptionsVector.
                removeElementAt(
                  COMMAND_LINE_ARCH_NAME_NUMBER-1 );
            }
          catch( ArrayIndexOutOfBoundsException e )
            {
              System.err.println( "checkAndGetArchiveName: " +
                                  "this shouldn\'t happen" );
            }
        /*end of try*/

          if (AllActions.verbose.flag)
            System.out.println( "archive name specified is " +
                                AllActions.archiveName );
          else
            ;
        /*end of if on AllActions.verbose.flag*/
        }
    /*end of if on commandLineOptionsVector.size*/

      return nameSpecified;
    }
/*end of checkAndGetArchiveName*/



  public
  static
  boolean

  analyzeAndGetOptions( Vector commandLineOptionsVector,
                        boolean gzipOptionSpecified       )
/*****************************************************************************/
/* Function name: analyzeAndGetOptions                                       */
/* File name: Jtar.java                                                      */
/* Purpose: to analyze command line arguments                                */
/* Parameters: commandLineOptionsVector - vector, containig strings          */
/*                                 of command line arguments to Jtar         */
/*             gzipOptionSpecified                                           */
/* Returns: true - if the options are OK, false - otherwise                  */
/* Last modified: 20.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      String options;
      boolean optionsAreOK = false;
      boolean allOptionsFitTogether = true;
      boolean allOptionsRecognized = true;
      boolean archiveNameSpecified = false;
                                              /* analyzeAndGetOptions */
/*---------------------------------------------------------------------------*/

      options = (String)
        commandLineOptionsVector.firstElement();

      checkVerbose( options,
                    commandLineOptionsVector );
      archiveNameSpecified =
        checkAndGetArchiveName( commandLineOptionsVector );

      optionsAreOK =
        archiveNameSpecified;

      try
        {
          commandLineOptionsVector.          /* remove options string    */
            removeElementAt( 0 );               /* we leave only file names */
        }
      catch( ArrayIndexOutOfBoundsException e )
        {
          System.err.println( "analyzeAndGetOptions: " +
                              "this shouldn\'t happen" );
        }
    /*end of try*/

      if (archiveNameSpecified)
        {
          allOptionsRecognized =                     /* set up options */
            AllActions.setUp( options );
          allOptionsFitTogether =                 /* check for conflicts */
            AllActions.
              optionsFitGoodTogetherSetMethods();

          optionsAreOK = optionsAreOK &
                           allOptionsFitTogether;
        }
      else
        ;
    /*end of if*/

      if (gzipOptionSpecified &&
          AllActions.verbose.flag)
        System.out.println( "\"" +
          java.lang.String.valueOf(
            COMMAND_LINE_OPTION_GZIP ) +
          "\" option specified (gzip)"   );
                             
      else
        ;
    /*end of if on AllActions.verbose.flag*/


      return optionsAreOK;

    }
/*end of analyzeAndGetOptions*/


  public
  static
  boolean

  actOnOptions( Vector commandLineOptionsVector )
/*****************************************************************************/
/* Function name: actOnOptions                                               */
/* File name: Jtar.java                                                      */
/* Purpose: to perform actual archiving work                                 */
/* Parameters: commandLineOptionsVector - vector, containig strings          */
/*                                 of command line arguments to Jtar         */
/* Returns: true - if everything is all rigth, false - otherwise             */
/* Last modified: 12.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      boolean result = true;
                                                       /* actOnOptions */
/*---------------------------------------------------------------------------*/

      if ( AllActions.
            whatToActUpon == null )
        System.err.println( "actOnOptions: " +
                              "no methods to call" );
      else
        result = AllActions.
                   whatToActUpon.
                     action( commandLineOptionsVector,
                             null,
                             null                         );
    /*end of if*/

      return result;

    }
/*end of actOnOptions*/



  public
  static
  void

  usageBasics()
/*****************************************************************************/
/* Function name: usageBasics                                                */
/* File name: Jtar.java                                                      */
/* Purpose: to print out a basic usage message on the standard output        */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 13.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                       /* usageBasics */
/*---------------------------------------------------------------------------*/

      System.out.print( "\nUsage: " );
      System.out.println( HELP_EXAMPLE_ENVOCATION_PREFIX +
                          "<options> [files]" );
      System.out.print( "You must specify only one " +
                        "of the following options: " );
      System.out.println(
        "\"" +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_TABLE ) +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_EXTRACT ) +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_CREATE ) +
        "\"" );
      System.out.println(
        "\"" +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_FILE ) +
        "\"" +
        " option also must be specified" );

    }
/*end of usageBasics*/



  public
  static
  void

  help()
/*****************************************************************************/
/* Function name: help                                                       */
/* File name: Jtar.java                                                      */
/* Purpose: to print out help message on the standard output                 */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 20.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                             /* help */
/*---------------------------------------------------------------------------*/

      usageBasics();

      System.out.println(
         "\n\"" +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_TABLE ) +
         "\"" +
         " prints out a list of file components," +
         " comprising a tar archive" );
      System.out.println(
         "\"" +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_EXTRACT ) +
         "\"" +
         " extracts files from a tar archive" );
      System.out.println(
        "\"" +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_CREATE ) +
        "\"" +
        " creates a tar archive" ); 
      System.out.println(
        "\"" +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_FILE ) +
        " <tar archive file name>" +
        "\"" +
        " is used to set the name " +
        "of the tar archive." +
        "\n    It must be the last option" );
      System.out.println(
        "\"" +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_VERBOSE ) +
        "\"" +
        " is used to print out " +
        "additional information" );
      System.out.println(
        "\"" +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_GZIP ) +
        "\"" +
        " is used to process a " +
        "GZIP compressed tar file" +
        "\n    " +
        "both by creation and by extraction" );

                                                          /* examples */
      System.out.println( "\nExamples:");
      System.out.println(
        HELP_EXAMPLE_ENVOCATION_PREFIX +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_TABLE ) +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_FILE ) +
        HELP_ARCHIVE_NAME_SUFFIX    );
      System.out.println( HELP_EXAMPLE_EMPTY_PREFIX +
                          "prints out a list of file components,\n" +
                          HELP_EXAMPLE_EMPTY_PREFIX +
                          "comprising the archive file,"           );
      System.out.println( HELP_EXAMPLE_EMPTY_PREFIX +
                          "named" +
                          HELP_ARCHIVE_NAME_SUFFIX );

      System.out.println(
        HELP_EXAMPLE_ENVOCATION_PREFIX +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_EXTRACT ) +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_FILE ) +
        HELP_ARCHIVE_NAME_SUFFIX    );
      System.out.println( HELP_EXAMPLE_EMPTY_PREFIX +
                          "extracts files from the archive" );

      System.out.println(
        HELP_EXAMPLE_ENVOCATION_PREFIX +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_CREATE ) +
        java.lang.String.valueOf(
          COMMAND_LINE_OPTION_FILE ) +
        HELP_ARCHIVE_NAME_SUFFIX +
        HELP_EXAMPLE_DIR_NAME_SUFFIX );
      System.out.println( HELP_EXAMPLE_EMPTY_PREFIX +
                          "creates the tar archive\n" +
                          HELP_EXAMPLE_EMPTY_PREFIX +
                          "from all files in the current directory" );

    }
/*end of help*/



  public
  static
  void

  usage()
/*****************************************************************************/
/* Function name: usage                                                      */
/* File name: Jtar.java                                                      */
/* Purpose: to print out usage message on the standard output                */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 20.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                          /* usage */
/*---------------------------------------------------------------------------*/

      usageBasics();
      System.out.println( "try\n" +
                          HELP_EXAMPLE_ENVOCATION_PREFIX +
                          COMMAND_LINE_OPTION_LONG_PREFIX +
                          COMMAND_LINE_OPTION_LONG_HELP   +
                          " for more information" );
      System.out.println(
        HELP_EXAMPLE_ENVOCATION_PREFIX +
        COMMAND_LINE_OPTION_LONG_PREFIX +
        COMMAND_LINE_OPTION_LONG_VERSION +
        " for the version number" );

    }
/*end of usage*/



  public
  static
  void

  main( String[] args )
/*****************************************************************************/
/* Function name: main                                                       */
/* File name: Jtar.java                                                      */
/* Purpose: main method for the Jtar class                                   */
/* Parameters: agrument strings array - see the usage method in this file    */
/* Returns:                                                                  */
/* Last modified: 20.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
      int i;

      Vector commandLineOptionsVector = new Vector( args.length );

      boolean actionPerformedOK = true;
      boolean helpOptionSpecified = false;
      boolean fileOptionSpecified = false;
      boolean gzipOptionSpecified = false;
      boolean versionOptionSpecified = false;
      boolean optionsAreOK = true;
                                                          /* main */
/*---------------------------------------------------------------------------*/

      if (args.length < 1)                      /* no args */
        usage();
      else
        ;
    /*end of if*/

      helpOptionSpecified =
        scanForTheLongHelpOption(
          args,
          commandLineOptionsVector );

      versionOptionSpecified =
        scanForTheLongVersionOption(
          commandLineOptionsVector );

      if ( !helpOptionSpecified &&
           (commandLineOptionsVector.size() >= 1) )
        {
        fileOptionSpecified =
          checkForCommandLineOptionFile(
            commandLineOptionsVector       );

        if (!fileOptionSpecified)
          {
            System.err.println(
              "\"" +
              java.lang.String.valueOf(
                COMMAND_LINE_OPTION_FILE ) +
              "\"" +
              " option is not specified\n" );
            help();
          }
        else
          ;
      /*end of if on fileOptionSpecified*/

        }
      else
        ;
    /*end of if on helpOptionSpecified*/

      if (helpOptionSpecified ||
          (commandLineOptionsVector.size() <= 1) ||
          !fileOptionSpecified   )
        ;                                              /* do nothing */
      else
        {
          gzipOptionSpecified =
            checkForCommandLineOptionGzip(
              commandLineOptionsVector       );

          optionsAreOK =
            analyzeAndGetOptions(
              commandLineOptionsVector,
              gzipOptionSpecified        );

          if (optionsAreOK)
            actionPerformedOK =
              actOnOptions( commandLineOptionsVector );
          else
            {
              System.err.println( "\nexiting due to an error!\n" );
              help();
            }
        /*end of if on optionsAreOK*/
        }
    /*end of if on helpOptionSpecified*/

      System.exit( Error.exitCode );

    }
/*end of main*/

} /*end of Jtar */

/*****************************************************************************/
/* End of file: Jtar.java                                                    */
/*****************************************************************************/

