/*****************************************************************************/
/* File name: MakeDirectories.java                                           */
/* Purpose: to implement MakeDirectories methods                             */
/* Last modified: 12.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* This software is distributed under the terms                              */
/* of the GNU General Public License as published                            */
/* by the Free Software Foundation; either version 2, or (at your option)    */
/* any later version.                                                        */
/* See file COPYING for your rights (GNU GPL)                                */
/*****************************************************************************/


package jtar;

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;
import general.*;

public
class

MakeDirectories

       implements general.Constants

{

  private final static String RCSID = "$Header: /d/JTAR03/jtar/RCS/MakeDirectories.java,v 1.4 1998/12/13 17:19:20 vklimov Exp $";


  public
  MakeDirectories()
/*****************************************************************************/
/* Function name: MakeDirectories                                            */
/* File name: MakeDirectories.java                                           */
/* Purpose: constructor for the MakeDirectories object                       */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 01.11.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                    /* MakeDirectories */
/*---------------------------------------------------------------------------*/

    }
/*end of MakeDirectories*/


  public
  static
  void
  printForMkdir( String pathname, int length,
                 Header tarHeader,
                 BufferRead readBuffer )
/*****************************************************************************/
/* Function name: printForMkdir                                              */
/* File name: MakeDirectories.java                                           */
/* Purpose: to print out a uniform line for each created directory           */
/* Parameters: pathname - the name of the created directory                  */
/*             length - length of the name (number of chars)                 */
/*             tarHeader                                                     */
/*             readBuffer                                                    */
/* Returns:                                                                  */
/* Last modified: 12.12.98                                                   */
/* Author of the Java translation: Victor Klimov, 1998                       */
/*****************************************************************************/
    {
      int i;
      int totalPrefixStatementStringLength;
      int emptyPrefixLentgth;

      String name;
      String prefixStatementString;
      DecimalFormat NumberFormat;
      StringBuffer emptySpaces;
                                                       /* printForMkdir */
/*---------------------------------------------------------------------------*/


      if (AllActions.blockNumberOption)
        {
          tarHeader.printBlockNumber( (Buffer) readBuffer );
        }
      else
        ;
    /*end of if on blockNumberOption*/

      name =
        util.QuoteString.quoteCopyString( pathname );

      prefixStatementString = new
        String( "Creating directory:" );

      totalPrefixStatementStringLength =
        tarHeader.ugswidth + tarHeader.DATEWIDTH;

      emptyPrefixLentgth =
        totalPrefixStatementStringLength -
        prefixStatementString.length();

      emptySpaces = new StringBuffer( emptyPrefixLentgth );
      for (i=0;i<emptyPrefixLentgth;i++)
        emptySpaces.append( ' ' );
    /*end of for-loop*/

      System.out.print( emptySpaces.toString() +
                        prefixStatementString    );
                          
      emptyPrefixLentgth =
        length - name.length();
      if (emptyPrefixLentgth < 0)
        emptyPrefixLentgth = 0;
      else
        ;
    /*end of if*/

      emptySpaces = new StringBuffer( emptyPrefixLentgth );
      for (i=0;i<emptyPrefixLentgth;i++)
        emptySpaces.append( ' ' );
    /*end of for-loop*/

      System.out.print( emptySpaces.toString() +
                        name                     );
      System.out.println( "" );
    }
/*end of printForMkdir*/


  public
  static
  boolean
  makeDirectories( String fileName,
                   Header tarHeader,
                   BufferRead readBuffer )
/*****************************************************************************/
/* Function name: makeDirectories                                            */
/* File name: MakeDirectories.java                                           */
/* Purpose: to check if there is a need to create intermediate directories   */
/*          and, if yes, to create them                                      */
/* Parameters: fileName                                                      */
/*             tarHeader                                                     */
/*             readBuffer                                                    */
/* Returns: true - if intermediate directories were created                  */
/*          false - otherwise                                                */
/* Last modified: 11.12.98                                                   */
/* Author of the Java translation: Victor Klimov, 1998                       */
/*****************************************************************************/
    {
      int cursor;                       /* points into path */
      boolean didSomething = false;        /* did we do anything yet? */
      boolean created;
      boolean continueToLoop = true;
      int status;
      int pathSeparatorLength;
      int trailingSeparatorIndexInPrefix = 0;
      int lengthPrefixTillCursor;
      String prefixTillCursor = null;
      String dirName;
      File dirFile;
                                               /* makeDirectories */
/*---------------------------------------------------------------------------*/

      pathSeparatorLength =
        File.separator.length();

      for (cursor = fileName.indexOf( File.separator );
           continueToLoop                                &&
           (cursor != general.Constants.JAVA_ERROR_CODE);
           cursor = fileName.
             indexOf( File.separator,
                      cursor + 1      )
          )
        {
                                             /* Avoid mkdir of empty string, */
                                             /* if leading or double '/'     */
          try
            {
              prefixTillCursor =
                fileName.substring( 0, cursor );
              trailingSeparatorIndexInPrefix =
                prefixTillCursor.lastIndexOf(
                  File.separator );

              if ( (cursor == 0) ||
                   (trailingSeparatorIndexInPrefix +
                    pathSeparatorLength == cursor)
                 )
                continue;                              /* continue ! */
              else
                ;
            /*end of if*/
            }
          catch( IndexOutOfBoundsException e)
            {
              System.out.println( "Should never happen" );
            }
        /*end of try-catch*/
                                                       /* Avoid mkdir where */
                                                       /* last part of path */
                                                       /* is '.'            */
          if (prefixTillCursor == null)
            return false;
          else
            ;
        /*end of if on null*/

          lengthPrefixTillCursor =
            prefixTillCursor.length();
          if ( (prefixTillCursor.                      /* last char */
                charAt( lengthPrefixTillCursor-1 ) ==
              CURRENT_DIRECTORY_ABBREVIATION) &&       /* . */
               ( (cursor == 1) ||
                   (trailingSeparatorIndexInPrefix +   /* . follows a / */
                    pathSeparatorLength + 1 == cursor)
                 )
             )
            continue;                                  /* continue ! */
          else
            ;
        /*end of if on . */

          dirName =                                 /* dir name to create */
            fileName.substring( 0, cursor );
          dirFile = new File( dirName );

          created = dirFile.mkdirs();

          if (created)
            {
              if (AllActions.verbose.flag)
                printForMkdir( dirName, dirName.length(),
                               tarHeader, readBuffer        );
              else
                ;
            /*end of if on verbose*/

              didSomething = true;

              continue;                                /* continue ! */
            }
          else /*nothing created*/
            ;
        /*end of if on created*/

          continueToLoop = false;

                                            /* Some other error in the mkdir */
                                            /* we return to the caller       */
          break;
        }
    /*end of for-loop*/

      return didSomething;

    }
/*end of makeDirectories*/


  public
  static
  void

  main( String[] args )
/*****************************************************************************/
/* Function name: main                                                       */
/* File name: MakeDirectories.java                                           */
/* Purpose: to test the MakeDirectories class                                */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 05.11.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                         /* main */
/*---------------------------------------------------------------------------*/


    }
/*end of main*/

} /*end of class MakeDirectories */

/*****************************************************************************/
/* End of file: MakeDirectories.java                                         */
/*****************************************************************************/
