/*****************************************************************************/
/* File name: TarWriteResource.java                                          */
/* Purpose: Output object to write to a generic tar stream                   */
/*                                                                           */
/* Last modified: 19.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/* For the list of authors and contributors of the original C version        */
/* of the GNU tar see files AUTHORS and THANKS                               */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* This software is distributed under the terms                              */
/* of the GNU General Public License as published                            */
/* by the Free Software Foundation; either version 2, or (at your option)    */
/* any later version.                                                        */
/* See file COPYING for your rights (GNU GPL)                                */
/*****************************************************************************/

package jtar;


class
TarWriteResource
{
  private final static String RCSID = "$Header: /d/JTAR10/jtar/RCS/TarWriteResource.java,v 1.1 1998/12/20 03:45:03 vklimov Exp $";


  TarWritable archiveStreamToUse;


  TarWriteResource( TarWritable archiveStreamToUse ) 
/*****************************************************************************/
/* Function name: TarWriteResource                                           */
/* File name: TarWriteResource.java                                          */
/* Purpose: constructor object to provide resources to write to a generic tar*/
/* Parameters: archiveStreamToUse                                            */
/* Returns:                                                                  */
/* Last modified: 18.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                       /* TarWriteResource */
/*---------------------------------------------------------------------------*/

      this.archiveStreamToUse = archiveStreamToUse;

    }
/*end of TarWriteResource*/



  void
  write( byte b[],
         int off,
         int len    ) throws java.io.IOException
/*****************************************************************************/
/* Function name: write                                                      */
/* File name: TarReadResource.java                                           */
/* Purpose: to write bytes from a bytes array                                */
/* Parameters: b - byte array to write the data from                         */
/*             off - offset in the byte array to begin to write from         */
/*             len - maximum number of bytes to write                        */
/* Returns:                                                                  */
/* Last modified: 19.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                            /* write */
/*---------------------------------------------------------------------------*/

      archiveStreamToUse.write( b, off, len );
    }
/*end of write*/


  void
  close() throws java.io.IOException
/*****************************************************************************/
/* Function name: close                                                      */
/* File name: TarReadResource.java                                           */
/* Purpose: to close generic tar stream                                      */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 18.12.98                                                   */
/* Author: Victor Klimov, 1998                                               */
/*****************************************************************************/
    {
                                                            /* close */
/*---------------------------------------------------------------------------*/

      archiveStreamToUse.close();
    }
/*end of close*/




}/*end of TarWriteResource class*/

/*****************************************************************************/
/* End of file: TarWriteResource.java                                        */
/*****************************************************************************/
